#!/bin/bash -e
# This script tests the base image and invokes the secure baseline audit
# If any issues occur, it exits with non-zero exit code
OSVER=$(grep Ubuntu /etc/issue | awk '{print $2}')
OSMAJORVER=$(echo $OSVER | cut -d "." -f 1)

function bail {
    echo "$@"
    exit 1
}

# BASE IMAGE TESTS
mdsd -V || bail "Error running mdsd -V. Build will now abort."
service mdsd start || echo Service is already started - continuing
sleep 1
# Don't care if this generates an error, we care if logs are generated
if [ ! -e /var/log/mdsd.err ]; then
    bail "No mdsd logs generated under /var/log. Build will now abort"
fi

service azsecd start || echo Service is already started - continuing
ps -C azsec-scan.py || bail "Azure security scanning service not started"
disabledScans=$(grep -L cycle=\"P /etc/azsec/cfg/*)
if [ -n "$disabledScans" ]; then
    bail "Scans are not enabled in scan file $disabledScans"
fi

fqdn=$(hostname -f) || bail "Error running hostname -f. Build will now abort."

echo "FQDN = $fqdn"
if [[ $fqdn != *internal.cloudapp.net ]] && [[ $fqdn != *internal.chinacloudapp.cn ]]; then
    bail "Hostname -f [$fqdn] returns an FQDN ending with neither 'internal.cloudapp.net' nor 'internal.chinacloudapp.cn'"
fi
grep 'options timeout:1 attempts:5' /etc/resolv.conf || bail "Retry options not found in /etc/resolv.conf"
egrep '^hosts:\s+files\s+cache' /etc/nsswitch.conf || bail "unscd not enabled in /etc/nsswitch.conf"
egrep '^\s*enable-cache\s+hosts\s+yes' /etc/nscd.conf || bail "Host caching not enabled in /etc/nscd.conf"

source /etc/profile
if [ -z $JAVA_HOME ] || [ ! -d $JAVA_HOME ]; then
    bail "JAVA_HOME [$JAVA_HOME] is not a valid directory. Build will now abort"
fi

ofilelimit=$(egrep -m 1 '^\*\s+hard\s+nofile' /etc/security/limits.conf | awk '{print $4}')
expectedfilelimit=10000
if [ -z "$ofilelimit" ]; then
    bail "No open file limit specified in /etc/security/limits.conf. Please specify a value of 10000 (nofile 10000)"
elif [ $ofilelimit -lt $expectedfilelimit ]; then
    bail "File limit [$ofilelimit] should be $expectedfilelimit"
else
    echo "File limit [$ofilelimit] is >= $expectedfilelimit"
fi

pamLimitCount=$(egrep -c "^session\s+required\s+pam_limits.so" /etc/pam.d/common-session)
if [ $pamLimitCount -lt 1 ]; then
    bail "No lines found in /etc/pam.d/common-session that match 'session required pam_limits.so'"
else
    echo "Session limits enabled in /etc/pam.d/common-session"
fi

# SECURE BASELINE TESTS
plaintextPass=$(egrep -c 'UserPassword>REDACTED|PublicKey' /var/lib/waagent/ovf-env.xml)
if [ $plaintextPass -lt 1 ] ; then
    bail "Password was not redacted from /var/lib/waagent/ovf-env.xml"
fi

# Following test applies only to 12.04. Cleartext password bug is fixed in 14.04.
filename=$(find /var/lib/cloud/instances -name obj.pkl | head -n 1)
if [ -n "$filename" ]  && [ $OSMAJORVER -eq 12 ]; then
    bail "$filename should be removed"
fi

# Confirm we comply with the baseline
results=$(azsec-scan.py -m -n bscan)
if ! echo $results | grep -q  "0 errors and 0 findings"; then
    echo $results | sed 's/|@|/\n/g' | grep -v PASS
    bail "Baseline security scan failed"
fi

# CLAMAV TEST
if [ ! -e /var/lib/clamav/main.cvd ]; then
    bail "Failed to download Clam AV signatures: /var/lib/clamav/main.cvd"
fi

echo "SUCCESS! All tests passed"
